@@ -1,43 +1,22 @@
-/****************************************************************************
-**
-** Copyright (C) 2021 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2021 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "viewtestutils_p.h"
 
 #include <QtCore/QRandomGenerator>
+#include <QtCore/QTimer>
 #include <QtQuick/QQuickView>
 #include <QtQuick/QQuickView>
 #include <QtGui/QScreen>
+#include <QtGui/private/qhighdpiscaling_p.h>
 #include <QtGui/qpa/qwindowsysteminterface.h>
 
 #include <QtTest/QTest>
 
 #include <QtQuick/private/qquickdeliveryagent_p_p.h>
+#if QT_CONFIG(quick_itemview)
 #include <QtQuick/private/qquickitemview_p_p.h>
+#endif
 #include <QtQuick/private/qquickwindow_p.h>
 
 #include <QtQuickTestUtils/private/visualtestutils_p.h>
@@ -74,37 +53,10 @@ void QQuickViewTestUtils::moveMouseAway(QQuickView *window)
 #endif
 }
 
-void QQuickViewTestUtils::moveAndRelease(QQuickView *window, const QPoint &position)
-{
-    QTest::mouseMove(window, position);
-    QTest::mouseRelease(window, Qt::LeftButton, {}, position);
-}
-
-void QQuickViewTestUtils::moveAndPress(QQuickView *window, const QPoint &position)
-{
-    QTest::mouseMove(window, position);
-    QTest::mousePress(window, Qt::LeftButton, {}, position);
-}
-
-void QQuickViewTestUtils::flick(QQuickView *window, const QPoint &from, const QPoint &to, int duration)
-{
-    const int pointCount = 5;
-    QPoint diff = to - from;
-
-    // send press, five equally spaced moves, and release.
-    moveAndPress(window, from);
-
-    for (int i = 0; i < pointCount; ++i)
-        QTest::mouseMove(window, from + (i+1)*diff/pointCount, duration / pointCount);
-
-    moveAndRelease(window, to);
-    QTest::qWait(50);
-}
-
 QList<int> QQuickViewTestUtils::adjustIndexesForAddDisplaced(const QList<int> &indexes, int index, int count)
 {
     QList<int> result;
-    for (int i=0; i<indexes.count(); i++) {
+    for (int i=0; i<indexes.size(); i++) {
         int num = indexes[i];
         if (num >= index) {
             num += count;
@@ -117,7 +69,7 @@ QList<int> QQuickViewTestUtils::adjustIndexesForAddDisplaced(const QList<int> &i
 QList<int> QQuickViewTestUtils::adjustIndexesForMove(const QList<int> &indexes, int from, int to, int count)
 {
     QList<int> result;
-    for (int i=0; i<indexes.count(); i++) {
+    for (int i=0; i<indexes.size(); i++) {
         int num = indexes[i];
         if (from < to) {
             if (num >= from && num < from + count)
@@ -138,7 +90,7 @@ QList<int> QQuickViewTestUtils::adjustIndexesForMove(const QList<int> &indexes,
 QList<int> QQuickViewTestUtils::adjustIndexesForRemoveDisplaced(const QList<int> &indexes, int index, int count)
 {
     QList<int> result;
-    for (int i=0; i<indexes.count(); i++) {
+    for (int i=0; i<indexes.size(); i++) {
         int num = indexes[i];
         if (num >= index)
             num -= count;
@@ -155,7 +107,7 @@ QQuickViewTestUtils::QaimModel::QaimModel(QObject *parent)
 int QQuickViewTestUtils::QaimModel::rowCount(const QModelIndex &parent) const
 {
     Q_UNUSED(parent);
-    return list.count();
+    return list.size();
 }
 
 int QQuickViewTestUtils::QaimModel::columnCount(const QModelIndex &parent) const
@@ -200,15 +152,15 @@ QString QQuickViewTestUtils::QaimModel::number(int index) const
 
 void QQuickViewTestUtils::QaimModel::addItem(const QString &name, const QString &number)
 {
-    emit beginInsertRows(QModelIndex(), list.count(), list.count());
+    emit beginInsertRows(QModelIndex(), list.size(), list.size());
     list.append(QPair<QString,QString>(name, number));
     emit endInsertRows();
 }
 
 void QQuickViewTestUtils::QaimModel::addItems(const QList<QPair<QString, QString> > &items)
 {
-    emit beginInsertRows(QModelIndex(), list.count(), list.count()+items.count()-1);
-    for (int i=0; i<items.count(); i++)
+    emit beginInsertRows(QModelIndex(), list.size(), list.size()+items.size()-1);
+    for (int i=0; i<items.size(); i++)
         list.append(QPair<QString,QString>(items[i].first, items[i].second));
     emit endInsertRows();
 }
@@ -222,8 +174,8 @@ void QQuickViewTestUtils::QaimModel::insertItem(int index, const QString &name,
 
 void QQuickViewTestUtils::QaimModel::insertItems(int index, const QList<QPair<QString, QString> > &items)
 {
-    emit beginInsertRows(QModelIndex(), index, index+items.count()-1);
-    for (int i=0; i<items.count(); i++)
+    emit beginInsertRows(QModelIndex(), index, index+items.size()-1);
+    for (int i=0; i<items.size(); i++)
         list.insert(index + i, QPair<QString,QString>(items[i].first, items[i].second));
     emit endInsertRows();
 }
@@ -265,7 +217,7 @@ void QQuickViewTestUtils::QaimModel::modifyItem(int idx, const QString &name, co
 
 void QQuickViewTestUtils::QaimModel::clear()
 {
-    int count = list.count();
+    int count = list.size();
     if (count > 0) {
         beginRemoveRows(QModelIndex(), 0, count-1);
         list.clear();
@@ -301,11 +253,11 @@ private:
 };
 
 void QQuickViewTestUtils::QaimModel::matchAgainst(const QList<QPair<QString, QString> > &other, const QString &error1, const QString &error2) {
-    for (int i=0; i<other.count(); i++) {
+    for (int i=0; i<other.size(); i++) {
         QVERIFY2(list.contains(other[i]),
                  ScopedPrintable(other[i].first + QLatin1Char(' ') + other[i].second + QLatin1Char(' ') + error1));
     }
-    for (int i=0; i<list.count(); i++) {
+    for (int i=0; i<list.size(); i++) {
         QVERIFY2(other.contains(list[i]),
                  ScopedPrintable(list[i].first + QLatin1Char(' ') + list[i].second + QLatin1Char(' ') + error2));
     }
@@ -362,7 +314,7 @@ bool QQuickViewTestUtils::ListRange::isValid() const
 
 int QQuickViewTestUtils::ListRange::count() const
 {
-    return indexes.count();
+    return indexes.size();
 }
 
 QList<QPair<QString,QString> > QQuickViewTestUtils::ListRange::getModelDataValues(const QaimModel &model)
@@ -370,7 +322,7 @@ QList<QPair<QString,QString> > QQuickViewTestUtils::ListRange::getModelDataValue
     QList<QPair<QString,QString> > data;
     if (!valid)
         return data;
-    for (int i=0; i<indexes.count(); i++)
+    for (int i=0; i<indexes.size(); i++)
         data.append(qMakePair(model.name(indexes[i]), model.number(indexes[i])));
     return data;
 }
@@ -416,12 +368,13 @@ void QQuickViewTestUtils::StressTestModel::updateModel()
     }
 }
 
+#if QT_CONFIG(quick_itemview)
 bool QQuickViewTestUtils::testVisibleItems(const QQuickItemViewPrivate *priv, bool *nonUnique, FxViewItem **failItem, int *expectedIdx)
 {
     QHash<QQuickItem*, int> uniqueItems;
 
     int skip = 0;
-    for (int i = 0; i < priv->visibleItems.count(); ++i) {
+    for (int i = 0; i < priv->visibleItems.size(); ++i) {
         FxViewItem *item = priv->visibleItems.at(i);
         if (!item) {
             *failItem = nullptr;
@@ -452,6 +405,7 @@ bool QQuickViewTestUtils::testVisibleItems(const QQuickItemViewPrivate *priv, bo
 
     return true;
 }
+#endif
 
 namespace QQuickTouchUtils {
 
@@ -500,6 +454,10 @@ namespace QQuickTest {
         }
         const QRect screenGeometry = view.screen()->availableGeometry();
         const QSize size = view.size();
+        if (view.width() == 0)
+            view.setWidth(100);
+        if (view.height() == 0)
+            view.setHeight(100);
         const QPoint offset = QPoint(size.width() / 2, size.height() / 2);
         view.setFramePosition(screenGeometry.center() - offset);
     #if QT_CONFIG(cursor) // Get the cursor out of the way.
@@ -540,50 +498,63 @@ namespace QQuickTest {
     static Qt::KeyboardModifiers pressedTabletModifiers = Qt::NoModifier;
 
     void pointerPress(const QPointingDevice *dev, QQuickWindow *window, int pointId, const QPoint &p,
-                      Qt::MouseButton button, Qt::KeyboardModifiers modifiers)
+                      Qt::MouseButton button, Qt::KeyboardModifiers modifiers, int delay)
     {
+        const auto defaultDelay = QTest::defaultMouseDelay();
         switch (dev->type()) {
         case QPointingDevice::DeviceType::Mouse:
         case QPointingDevice::DeviceType::TouchPad:
-            QTest::mousePress(window, button, modifiers, p);
+            QTest::mousePress(window, button, modifiers, p, delay >= 0 ? delay : defaultDelay ? defaultDelay : 1);
             break;
         case QPointingDevice::DeviceType::TouchScreen:
+            // TODO apply delay when QTBUG-95421 is fixed
             QTest::touchEvent(window, const_cast<QPointingDevice *>(dev)).press(pointId, p, window);
             QQuickTouchUtils::flush(window);
             break;
         case QPointingDevice::DeviceType::Puck:
         case QPointingDevice::DeviceType::Stylus:
-        case QPointingDevice::DeviceType::Airbrush:
-            QTest::lastMouseTimestamp += QTest::defaultMouseDelay();
+        case QPointingDevice::DeviceType::Airbrush:{
+            const QPointF nativeLocal = QHighDpi::toNativeLocalPosition(p, window);
+            const QPointF nativeGlobal = QHighDpi::toNativeGlobalPosition(window->mapToGlobal(p), window);
+            QTest::lastMouseTimestamp += delay >= 0 ? delay : defaultDelay ? defaultDelay : 1;
             pressedTabletButton = button;
             pressedTabletModifiers = modifiers;
-            QWindowSystemInterface::handleTabletEvent(window, QTest::lastMouseTimestamp, dev, p, window->mapToGlobal(p),
+            QWindowSystemInterface::handleTabletEvent(window, QTest::lastMouseTimestamp, dev, nativeLocal, nativeGlobal,
                                                       button, 0.8, 0, 0, 0, 0, 0, modifiers);
             break;
+        }
         default:
             qWarning() << "can't send a press event from" << dev;
             break;
         }
     }
 
-    void pointerMove(const QPointingDevice *dev, QQuickWindow *window, int pointId, const QPoint &p)
+    void pointerMove(const QPointingDevice *dev, QQuickWindow *window, int pointId, const QPoint &p, int delay)
     {
+        const auto defaultDelay = QTest::defaultMouseDelay();
         switch (dev->type()) {
         case QPointingDevice::DeviceType::Mouse:
         case QPointingDevice::DeviceType::TouchPad:
-            QTest::mouseMove(window, p);
+            QTest::mouseMove(window, p, delay >= 0 ? delay : defaultDelay ? defaultDelay : 1);
             break;
         case QPointingDevice::DeviceType::TouchScreen:
+            // TODO apply delay when QTBUG-95421 is fixed
             QTest::touchEvent(window, const_cast<QPointingDevice *>(dev)).move(pointId, p, window);
             QQuickTouchUtils::flush(window);
             break;
         case QPointingDevice::DeviceType::Puck:
         case QPointingDevice::DeviceType::Stylus:
-        case QPointingDevice::DeviceType::Airbrush:
-            QTest::lastMouseTimestamp += QTest::defaultMouseDelay();
-            QWindowSystemInterface::handleTabletEvent(window, QTest::lastMouseTimestamp, dev, p, window->mapToGlobal(p),
-                                                      pressedTabletButton, 0, 0, 0, 0, 0, 0, pressedTabletModifiers);
+        case QPointingDevice::DeviceType::Airbrush: {
+            const QPointF nativeLocal = QHighDpi::toNativeLocalPosition(p, window);
+            const QPointF nativeGlobal = QHighDpi::toNativeGlobalPosition(window->mapToGlobal(p), window);
+            const auto delay = QTest::defaultMouseDelay();
+            // often QTest::defaultMouseDelay() == 0; but avoid infinite velocity
+            QTest::lastMouseTimestamp += delay >= 0 ? delay : defaultDelay ? defaultDelay : 1;
+            QWindowSystemInterface::handleTabletEvent(window, QTest::lastMouseTimestamp, dev, nativeLocal, nativeGlobal,
+                                                      pressedTabletButton, pressedTabletButton == Qt::NoButton ? 0 : 0.75,
+                                                      0, 0, 0, 0, 0, pressedTabletModifiers);
             break;
+        }
         default:
             qWarning() << "can't send a move event from" << dev;
             break;
@@ -591,30 +562,66 @@ namespace QQuickTest {
     }
 
     void pointerRelease(const QPointingDevice *dev, QQuickWindow *window, int pointId, const QPoint &p,
-                        Qt::MouseButton button, Qt::KeyboardModifiers modifiers)
+                        Qt::MouseButton button, Qt::KeyboardModifiers modifiers, int delay)
     {
+        const auto defaultDelay = QTest::defaultMouseDelay();
         switch (dev->type()) {
         case QPointingDevice::DeviceType::Mouse:
         case QPointingDevice::DeviceType::TouchPad:
-            QTest::mouseRelease(window, button, modifiers, p);
+            QTest::mouseRelease(window, button, modifiers, p, delay >= 0 ? delay : defaultDelay ? defaultDelay : 1);
             break;
         case QPointingDevice::DeviceType::TouchScreen:
+            // TODO apply delay when QTBUG-95421 is fixed
             QTest::touchEvent(window, const_cast<QPointingDevice *>(dev)).release(pointId, p, window);
             QQuickTouchUtils::flush(window);
             break;
         case QPointingDevice::DeviceType::Puck:
         case QPointingDevice::DeviceType::Stylus:
-        case QPointingDevice::DeviceType::Airbrush:
-            QTest::lastMouseTimestamp += QTest::defaultMouseDelay();
-            QWindowSystemInterface::handleTabletEvent(window, QTest::lastMouseTimestamp, dev, p, window->mapToGlobal(p),
+        case QPointingDevice::DeviceType::Airbrush: {
+            const QPointF nativeLocal = QHighDpi::toNativeLocalPosition(p, window);
+            const QPointF nativeGlobal = QHighDpi::toNativeGlobalPosition(window->mapToGlobal(p), window);
+            QTest::lastMouseTimestamp += delay >= 0 ? delay : defaultDelay ? defaultDelay : 1;
+            QWindowSystemInterface::handleTabletEvent(window, QTest::lastMouseTimestamp, dev, nativeLocal, nativeGlobal,
                                                       Qt::NoButton, 0, 0, 0, 0, 0, 0, modifiers);
             break;
+        }
         default:
             qWarning() << "can't send a press event from" << dev;
             break;
         }
     }
 
+    void pointerMoveAndPress(const QPointingDevice *dev, QQuickWindow *window,
+                             int pointId, const QPoint &p, Qt::MouseButton button,
+                             Qt::KeyboardModifiers modifiers, int delay)
+    {
+        pointerMove(dev, window, pointId, p, delay);
+        pointerPress(dev, window, pointId, p, button, modifiers);
+    }
+
+    void pointerMoveAndRelease(const QPointingDevice *dev, QQuickWindow *window,
+                               int pointId, const QPoint &p, Qt::MouseButton button,
+                               Qt::KeyboardModifiers modifiers, int delay)
+    {
+        pointerMove(dev, window, pointId, p, delay);
+        pointerRelease(dev, window, pointId, p, button, modifiers);
+    }
+
+    void pointerFlick(const QPointingDevice *dev, QQuickWindow *window,
+                      int pointId, const QPoint &from, const QPoint &to, int duration,
+                      Qt::MouseButton button, Qt::KeyboardModifiers modifiers, int delay)
+    {
+        const int pointCount = 5;
+        const QPoint diff = to - from;
+
+        // send press, five equally spaced moves, and release.
+        pointerMoveAndPress(dev, window, pointId, from, button, modifiers, delay);
+
+        for (int i = 0; i < pointCount; ++i)
+            pointerMove(dev, window, pointId, from + (i + 1) * diff / pointCount, duration / pointCount);
+
+        pointerMoveAndRelease(dev, window, pointId, to, button, modifiers);
+    }
 }
 
 QT_END_NAMESPACE
